@@ -1,12 +1,7 @@
-use crate::content::ser::{Content, ContentSerializer};
-use crate::{DeserializeAs, SerializeAs};
-use serde::de::{DeserializeSeed, EnumAccess, Error, MapAccess, SeqAccess, VariantAccess, Visitor};
-use serde::ser::{
-    Impossible, SerializeMap, SerializeSeq, SerializeStructVariant, SerializeTupleVariant,
+use crate::{
+    content::ser::{Content, ContentSerializer},
+    prelude::*,
 };
-use serde::{ser, Deserialize, Deserializer, Serialize, Serializer};
-use std::fmt;
-use std::marker::PhantomData;
 
 /// Represent a list of enum values as a map.
 ///
@@ -24,7 +19,7 @@ use std::marker::PhantomData;
 /// ```rust
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
-/// use serde_with::EnumMap;
+/// use serde_with::{serde_as, EnumMap};
 ///
 /// # #[derive(Debug, Clone, PartialEq, Eq)]
 /// #[derive(Serialize, Deserialize)]
@@ -40,7 +35,7 @@ use std::marker::PhantomData;
 ///     },
 /// }
 ///
-/// #[serde_with::serde_as]
+/// #[serde_as]
 /// # #[derive(Debug, Clone, PartialEq, Eq)]
 /// #[derive(Serialize, Deserialize)]
 /// struct VecEnumValues (
@@ -102,7 +97,7 @@ use std::marker::PhantomData;
 /// ```
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
-/// use serde_with::EnumMap;
+/// use serde_with::{serde_as, EnumMap};
 ///
 /// # #[derive(Debug, Clone, PartialEq, Eq)]
 /// #[derive(Serialize, Deserialize)]
@@ -112,7 +107,7 @@ use std::marker::PhantomData;
 ///     Unit,
 /// }
 ///
-/// #[serde_with::serde_as]
+/// #[serde_as]
 /// # #[derive(Debug, Clone, PartialEq, Eq)]
 /// #[derive(Serialize, Deserialize)]
 /// struct VecEnumValues {
@@ -136,17 +131,18 @@ use std::marker::PhantomData;
 /// // into this XML document
 /// // Duplicate keys are emitted for identical enum variants.
 /// let expected = r#"
+/// <?xml version="1.0" encoding="UTF-8"?>
 /// <VecEnumValues>
 ///     <vec>
 ///         <Int>123</Int>
 ///         <String>FooBar</String>
 ///         <Int>456</Int>
 ///         <String>XXX</String>
-///         <Unit></Unit>
+///         <Unit />
 ///     </vec>
 /// </VecEnumValues>"#
 /// // Remove whitespace
-/// .replace(' ', "")
+/// .replace("    ", "")
 /// .replace('\n', "");
 ///
 /// // Both serialization and deserialization work flawlessly.
@@ -156,7 +152,6 @@ use std::marker::PhantomData;
 /// assert_eq!(values, deserialized);
 /// # }
 /// ```
-#[derive(Debug, Copy, Clone)]
 pub struct EnumMap;
 
 impl<T> SerializeAs<Vec<T>> for EnumMap
@@ -179,7 +174,10 @@ where
     where
         D: Deserializer<'de>,
     {
-        struct EnumMapVisitor<T>(PhantomData<T>);
+        struct EnumMapVisitor<T> {
+            is_human_readable: bool,
+            phantom: PhantomData<T>,
+        }
 
         impl<'de, T> Visitor<'de> for EnumMapVisitor<T>
         where
@@ -188,15 +186,22 @@ where
             type Value = Vec<T>;
 
             fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
-                write!(formatter, "a map or enum values")
+                formatter.write_str("a map of enum values")
             }
 
             fn visit_map<A: MapAccess<'de>>(self, map: A) -> Result<Self::Value, A::Error> {
-                Vec::deserialize(SeqDeserializer(map))
+                Vec::deserialize(SeqDeserializer {
+                    delegate: map,
+                    is_human_readable: self.is_human_readable,
+                })
             }
         }
 
-        deserializer.deserialize_map(EnumMapVisitor(PhantomData))
+        let is_human_readable = deserializer.is_human_readable();
+        deserializer.deserialize_map(EnumMapVisitor {
+            is_human_readable,
+            phantom: PhantomData,
+        })
     }
 }
 
@@ -225,87 +230,91 @@ where
     type SerializeStruct = Impossible<S::Ok, S::Error>;
     type SerializeStructVariant = Impossible<S::Ok, S::Error>;
 
+    fn is_human_readable(&self) -> bool {
+        self.0.is_human_readable()
+    }
+
     fn serialize_bool(self, _v: bool) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i8(self, _v: i8) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i16(self, _v: i16) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i32(self, _v: i32) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i64(self, _v: i64) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i128(self, _v: i128) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u8(self, _v: u8) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u16(self, _v: u16) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u32(self, _v: u32) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u64(self, _v: u64) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u128(self, _v: u128) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_f32(self, _v: f32) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_f64(self, _v: f64) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_char(self, _v: char) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_str(self, _v: &str) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_bytes(self, _v: &[u8]) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_none(self) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
-    fn serialize_some<T: ?Sized>(self, _value: &T) -> Result<Self::Ok, Self::Error>
+    fn serialize_some<T>(self, _value: &T) -> Result<Self::Ok, Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_unit(self) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_unit_struct(self, _name: &'static str) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_unit_variant(
@@ -314,21 +323,21 @@ where
         _variant_index: u32,
         _variant: &'static str,
     ) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
-    fn serialize_newtype_struct<T: ?Sized>(
+    fn serialize_newtype_struct<T>(
         self,
         _name: &'static str,
         _value: &T,
     ) -> Result<Self::Ok, Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
-    fn serialize_newtype_variant<T: ?Sized>(
+    fn serialize_newtype_variant<T>(
         self,
         _name: &'static str,
         _variant_index: u32,
@@ -336,9 +345,9 @@ where
         _value: &T,
     ) -> Result<Self::Ok, Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_seq(self, len: Option<usize>) -> Result<Self::SerializeSeq, Self::Error> {
@@ -352,7 +361,7 @@ where
     }
 
     fn serialize_tuple(self, _len: usize) -> Result<Self::SerializeTuple, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_tuple_struct(
@@ -360,7 +369,7 @@ where
         _name: &'static str,
         _len: usize,
     ) -> Result<Self::SerializeTupleStruct, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_tuple_variant(
@@ -370,11 +379,11 @@ where
         _variant: &'static str,
         _len: usize,
     ) -> Result<Self::SerializeTupleVariant, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_map(self, _len: Option<usize>) -> Result<Self::SerializeMap, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_struct(
@@ -382,7 +391,7 @@ where
         _name: &'static str,
         _len: usize,
     ) -> Result<Self::SerializeStruct, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_struct_variant(
@@ -392,13 +401,13 @@ where
         _variant: &'static str,
         _len: usize,
     ) -> Result<Self::SerializeStructVariant, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 }
 
 /// Serialize a single element but turn the sequence into a map logic.
 ///
-/// It uses [`SerializeEnumAsMapElement`] for the map element serialization.
+/// It uses [`EnumAsMapElementSerializer`] for the map element serialization.
 ///
 /// The [`Serializer`] implementation handles all the `serialize_*_variant` functions and defers to [`SerializeVariant`] for the more complicated tuple and struct variants.
 struct SerializeSeqElement<M> {
@@ -413,9 +422,9 @@ where
     type Ok = M::Ok;
     type Error = M::Error;
 
-    fn serialize_element<T: ?Sized>(&mut self, value: &T) -> Result<(), Self::Error>
+    fn serialize_element<T>(&mut self, value: &T) -> Result<(), Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
         value.serialize(EnumAsMapElementSerializer {
             delegate: &mut self.delegate,
@@ -449,87 +458,91 @@ where
     type SerializeStruct = Impossible<Self::Ok, Self::Error>;
     type SerializeStructVariant = SerializeVariant<'a, M>;
 
+    fn is_human_readable(&self) -> bool {
+        self.is_human_readable
+    }
+
     fn serialize_bool(self, _v: bool) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i8(self, _v: i8) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i16(self, _v: i16) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i32(self, _v: i32) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i64(self, _v: i64) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_i128(self, _v: i128) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u8(self, _v: u8) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u16(self, _v: u16) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u32(self, _v: u32) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u64(self, _v: u64) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_u128(self, _v: u128) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_f32(self, _v: f32) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_f64(self, _v: f64) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_char(self, _v: char) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_str(self, _v: &str) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_bytes(self, _v: &[u8]) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_none(self) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
-    fn serialize_some<T: ?Sized>(self, _value: &T) -> Result<Self::Ok, Self::Error>
+    fn serialize_some<T>(self, _value: &T) -> Result<Self::Ok, Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_unit(self) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_unit_struct(self, _name: &'static str) -> Result<Self::Ok, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_unit_variant(
@@ -542,18 +555,18 @@ where
         Ok(())
     }
 
-    fn serialize_newtype_struct<T: ?Sized>(
+    fn serialize_newtype_struct<T>(
         self,
         _name: &'static str,
         _value: &T,
     ) -> Result<Self::Ok, Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
-    fn serialize_newtype_variant<T: ?Sized>(
+    fn serialize_newtype_variant<T>(
         self,
         _name: &'static str,
         _variant_index: u32,
@@ -561,18 +574,18 @@ where
         value: &T,
     ) -> Result<Self::Ok, Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
         self.delegate.serialize_entry(variant, value)?;
         Ok(())
     }
 
     fn serialize_seq(self, _len: Option<usize>) -> Result<Self::SerializeSeq, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_tuple(self, _len: usize) -> Result<Self::SerializeTuple, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_tuple_struct(
@@ -580,7 +593,7 @@ where
         _name: &'static str,
         _len: usize,
     ) -> Result<Self::SerializeTupleStruct, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_tuple_variant(
@@ -599,7 +612,7 @@ where
     }
 
     fn serialize_map(self, _len: Option<usize>) -> Result<Self::SerializeMap, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_struct(
@@ -607,7 +620,7 @@ where
         _name: &'static str,
         _len: usize,
     ) -> Result<Self::SerializeStruct, Self::Error> {
-        Err(ser::Error::custom("wrong type for EnumMap"))
+        Err(SerError::custom("wrong type for EnumMap"))
     }
 
     fn serialize_struct_variant(
@@ -636,7 +649,7 @@ struct SerializeVariant<'a, M> {
     content: Content,
 }
 
-impl<'a, M> SerializeStructVariant for SerializeVariant<'a, M>
+impl<M> SerializeStructVariant for SerializeVariant<'_, M>
 where
     M: SerializeMap,
 {
@@ -644,13 +657,9 @@ where
 
     type Error = M::Error;
 
-    fn serialize_field<T: ?Sized>(
-        &mut self,
-        key: &'static str,
-        value: &T,
-    ) -> Result<(), Self::Error>
+    fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<(), Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
         // Serialize to a Content type first
         let value: Content = value.serialize(ContentSerializer::new(self.is_human_readable))?;
@@ -665,7 +674,7 @@ where
     }
 }
 
-impl<'a, M> SerializeTupleVariant for SerializeVariant<'a, M>
+impl<M> SerializeTupleVariant for SerializeVariant<'_, M>
 where
     M: SerializeMap,
 {
@@ -673,9 +682,9 @@ where
 
     type Error = M::Error;
 
-    fn serialize_field<T: ?Sized>(&mut self, value: &T) -> Result<(), Self::Error>
+    fn serialize_field<T>(&mut self, value: &T) -> Result<(), Self::Error>
     where
-        T: Serialize,
+        T: Serialize + ?Sized,
     {
         // Serialize to a Content type first
         let value: Content = value.serialize(ContentSerializer::new(self.is_human_readable))?;
@@ -695,7 +704,10 @@ where
 /// Deserialize the sequence of enum instances.
 ///
 /// The main [`Deserializer`] implementation handles the outer sequence (e.g., `Vec`), while the [`SeqAccess`] implementation is responsible for the inner elements.
-struct SeqDeserializer<M>(M);
+struct SeqDeserializer<M> {
+    delegate: M,
+    is_human_readable: bool,
+}
 
 impl<'de, M> Deserializer<'de> for SeqDeserializer<M>
 where
@@ -703,6 +715,10 @@ where
 {
     type Error = M::Error;
 
+    fn is_human_readable(&self) -> bool {
+        self.is_human_readable
+    }
+
     fn deserialize_seq<V>(self, visitor: V) -> Result<V::Value, Self::Error>
     where
         V: Visitor<'de>,
@@ -734,7 +750,15 @@ where
     where
         T: DeserializeSeed<'de>,
     {
-        match seed.deserialize(EnumDeserializer(&mut self.0)) {
+        // We need to check if the map is done, or if there are still remaining elements.
+        // But we cannot ask the MapAccess directly.
+        // The only way is trying to deserialize, but we don't know the type yet.
+        // So we assume there is a value and try to deserialize it.
+        // If we later on find out that there is no value, we return a special error value, which we turn into `None`.
+        match seed.deserialize(EnumDeserializer {
+            delegate: &mut self.delegate,
+            is_human_readable: self.is_human_readable,
+        }) {
             Ok(value) => Ok(Some(value)),
             Err(err) => {
                 // Unfortunately we loose the optional aspect of MapAccess, so we need to special case an error value to mark the end of the map.
@@ -748,7 +772,7 @@ where
     }
 
     fn size_hint(&self) -> Option<usize> {
-        self.0.size_hint()
+        self.delegate.size_hint()
     }
 }
 
@@ -757,7 +781,10 @@ where
 /// The [`Deserializer`] implementation is the starting point, which first calls the [`EnumAccess`] methods.
 /// The [`EnumAccess`] is used to deserialize the enum variant type of the enum.
 /// The [`VariantAccess`] is used to deserialize the value part of the enum.
-struct EnumDeserializer<M>(M);
+struct EnumDeserializer<M> {
+    delegate: M,
+    is_human_readable: bool,
+}
 
 impl<'de, M> Deserializer<'de> for EnumDeserializer<M>
 where
@@ -765,6 +792,10 @@ where
 {
     type Error = M::Error;
 
+    fn is_human_readable(&self) -> bool {
+        self.is_human_readable
+    }
+
     fn deserialize_any<V>(self, visitor: V) -> Result<V::Value, Self::Error>
     where
         V: Visitor<'de>,
@@ -802,11 +833,11 @@ where
     where
         T: DeserializeSeed<'de>,
     {
-        match self.0.next_key_seed(seed)? {
+        match self.delegate.next_key_seed(seed)? {
             Some(key) => Ok((key, self)),
 
             // Unfortunately we loose the optional aspect of MapAccess, so we need to special case an error value to mark the end of the map.
-            None => Err(Error::custom(END_OF_MAP_IDENTIFIER)),
+            None => Err(DeError::custom(END_OF_MAP_IDENTIFIER)),
         }
     }
 }
@@ -818,21 +849,22 @@ where
     type Error = M::Error;
 
     fn unit_variant(mut self) -> Result<(), Self::Error> {
-        self.0.next_value()
+        self.delegate.next_value()
     }
 
     fn newtype_variant_seed<T>(mut self, seed: T) -> Result<T::Value, Self::Error>
     where
         T: DeserializeSeed<'de>,
     {
-        self.0.next_value_seed(seed)
+        self.delegate.next_value_seed(seed)
     }
 
     fn tuple_variant<V>(mut self, len: usize, visitor: V) -> Result<V::Value, Self::Error>
     where
         V: Visitor<'de>,
     {
-        self.0.next_value_seed(SeedTupleVariant { len, visitor })
+        self.delegate
+            .next_value_seed(SeedTupleVariant { len, visitor })
     }
 
     fn struct_variant<V>(
@@ -843,7 +875,7 @@ where
     where
         V: Visitor<'de>,
     {
-        self.0.next_value_seed(SeedStructVariant { visitor })
+        self.delegate.next_value_seed(SeedStructVariant { visitor })
     }
 }
 
